<?php

/**
 *
 * Copyright (c) 2015 Stokersoft (info@stokersoft.com)
 * 
 * This payment module is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 * 
 * This payment module is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * Version 2.0
 */
include ('quickpay_common.inc');

class osC_Payment_quickpay extends osC_Payment {

    var $_title, $_code = 'quickpay', $_status = false, $_sort_order, $_order_id, $_ignore_order_totals = array('sub_total', 'tax', 'total'), $_transaction_response, $pfHost = '';

    // class constructor
    function osC_Payment_quickpay() {
        global $osC_Database, $osC_Language, $osC_ShoppingCart;
        define('PF_DEBUG', MODULE_PAYMENT_QUICKPAY_DEBUG);
        $this->_title = $osC_Language->get('payment_quickpay_title');
        $this->_method_title = $osC_Language->get('payment_quickpay_method_title');
        $this->_sort_order = MODULE_PAYMENT_QUICKPAY_SORT_ORDER;
        $this->_status = ((MODULE_PAYMENT_QUICKPAY_STATUS == '1') ? true : false);



        $this->pfHost = 'payment.quickpay.net/';
        $this->form_action_url = 'https://' . $this->pfHost;

        if ($this->_status === true) {
            $this->order_status = MODULE_PAYMENT_QUICKPAY_ORDER_STATUS_ID > 0 ? (int)
                    MODULE_PAYMENT_QUICKPAY_ORDER_STATUS_ID : (int) ORDERS_STATUS_PAID;

            if ((int) MODULE_PAYMENT_QUICKPAY_ZONE > 0) {
                $check_flag = false;

                $Qcheck = $osC_Database->query('select zone_id from :table_zones_to_geo_zones where geo_zone_id = :geo_zone_id and zone_country_id = :zone_country_id order by zone_id');
                $Qcheck->bindTable(':table_zones_to_geo_zones', TABLE_ZONES_TO_GEO_ZONES);
                $Qcheck->bindInt(':geo_zone_id', MODULE_PAYMENT_QUICKPAY_ZONE);
                $Qcheck->bindInt(':zone_country_id', $osC_ShoppingCart->getBillingAddress('country_id'));
                $Qcheck->execute();

                while ($Qcheck->next()) {
                    if ($Qcheck->valueInt('zone_id') < 1) {
                        $check_flag = true;
                        break;
                    } elseif ($Qcheck->valueInt('zone_id') == $osC_ShoppingCart->getBillingAddress('zone_id')) {
                        $check_flag = true;
                        break;
                    }
                }

                if ($check_flag == false) {
                    $this->_status = false;
                }
            }
        }
    }

    function selection() {
        return array('id' => $this->_code, 'module' => $this->_method_title);
    }

    function pre_confirmation_check() {
        global $osC_ShoppingCart;

        $cart_id = $osC_ShoppingCart->getCartID();
        if (empty($cart_id)) {
            $osC_ShoppingCart->generateCartID();
        }
    }

    function confirmation() {
        $this->_order_id = osC_Order::insert(MODULE_PAYMENT_QUICKPAY_PROCESSING_ORDER_STATUS_ID);
    }

    function process_button() {
        global $osC_Customer, $osC_Currencies, $osC_ShoppingCart, $osC_Tax, $osC_Language;

        $currency = $osC_Currencies->getCode();

        $process_button_string = '';

        $language = MODULE_PAYMENT_QUICKPAY_LANGUAGE;
        if ($language == "English") {
            $language = "en";
        }
        if ($language == "Danish") {
            $language = "da";
        }

        if ($language == "German") {
            $language = "de";
        }

        if ($language == "Faeroese") {
            $language = "fo";
        }

        if ($language == "French") {
            $language = "fr";
        }


        if ($language == "Greenlandish") {
            $language = "gl";
        }

        if ($language == "Dutch") {
            $language = "nl";
        }

        if ($language == "Polish") {
            $language = "pl";
        }

        if ($language == "Swedish") {
            $language = "se";
        }


        $ordernumber = $this->order_prefix . substr('0000' . $this->_order_id, -4 +
                        strlen($this->order_prefix));

        $amount = $osC_ShoppingCart->getTotal();
        $amount = number_format($amount, 2);
        $amount = str_replace('.', '', $amount);
        $amount = str_replace(',', '', $amount);
        $autocapture = MODULE_PAYMENT_QUICKPAY_AUTOCAPTURE;
        if ($autocapture == "Yes") {
            $autocapture = 1;
        } else {
            $autocapture = 0;
        }

        $splitpayment = MODULE_PAYMENT_QUICKPAY_SPLITPAYMENT;
        if ($splitpayment == "Yes") {
            $splitpayment = 1;
        } else {
            $splitpayment = 0;
        }


        $payment_method = MODULE_PAYMENT_QUICKPAY_PAYMENT_METHOD;
        $prefix = MODULE_PAYMENT_QUICKPAY_ORDER_PREFIX;



        $params = array(
            "version" => "v10",
            "merchant_id" => MODULE_PAYMENT_QUICKPAY_MERCHANT_ID,
            "agreement_id" => MODULE_PAYMENT_QUICKPAY_AGREEMENT_ID,
            "order_id" => MODULE_PAYMENT_QUICKPAY_ORDER_PREFIX . $ordernumber,
            "amount" => $amount,
            "currency" => $currency,
            "continueurl" => HTTPS_SERVER . DIR_WS_HTTPS_CATALOG . FILENAME_CHECKOUT . '?process',
            "cancelurl" => HTTPS_SERVER . DIR_WS_HTTPS_CATALOG . FILENAME_CHECKOUT . '?checkout',
            "callbackurl" => HTTPS_SERVER . DIR_WS_HTTPS_CATALOG . FILENAME_CHECKOUT . '?callback&module=' . $this->_code,
            "language" => $language,
            "autocapture" => $autocapture,
            "autofee" => MODULE_PAYMENT_QUICKPAY_AUTOFEE,
            "payment_methods" => $payment_method,
            "branding_id" => MODULE_PAYMENT_QUICKPAY_BRANDING_ID,
            "google_analytics_tracking_id" => MODULE_PAYMENT_QUICKPAY_GOOGLE_TRACKING_ID,
            "google_analytics_client_id" => MODULE_PAYMENT_QUICKPAY_GOOGLE_ANALYTICS_CLIENT_ID,
            "product_id" => MODULE_PAYMENT_QUICKPAY_PAII_PRODUCT_ID,
            "category" => MODULE_PAYMENT_QUICKPAY_PAII_CATEGORY_ID
        );

        $secret = MODULE_PAYMENT_QUICKPAY_MERCHANT_KEY;

        $params["checksum"] = $this->sign($params, $secret);

        if ($osC_ShoppingCart->hasShippingAddress()) {
            $params['name_first'] = $osC_ShoppingCart->getShippingAddress('firstname');
            $params['name_last'] = $osC_ShoppingCart->getShippingAddress('lastname');
        } else {
            $params['name_first'] = $osC_ShoppingCart->getBillingAddress('firstname');
            $params['name_last'] = $osC_ShoppingCart->getBillingAddress('lastname');
        }

        $params['m_payment_id'] = $this->_order_id;
        $params['item_name'] = STORE_NAME . ' #' . $this->_order_id;
        $params['item_description'] = MODULE_PAYMENT_QUICKPAY_ITEM_DESCRIPTION;
        $params['custom_str1'] = $osC_Customer->getID();
        $process_button_string = '';

        $secureString = '';
        foreach ($params as $k => $v) {
            $secureString .= $k . '=' . urlencode(trim($v)) . '&';
        }
        $secureString = substr($secureString, 0, -1);
        foreach ($params as $key => $value) {
            $process_button_string .= osc_draw_hidden_field($key, $value);
        }
        return $process_button_string;
    }

    function process() {
        global $osC_ShoppingCart, $osC_Database;

        $prep = explode('-', $_SESSION['prepOrderID']);
        if ($prep[0] == $osC_ShoppingCart->getCartID()) {
            $Qcheck = $osC_Database->query('select orders_status_id from :table_orders_status_history where orders_id = :orders_id');
            $Qcheck->bindTable(':table_orders_status_history', TABLE_ORDERS_STATUS_HISTORY);
            $Qcheck->bindInt(':orders_id', $prep[1]);
            $Qcheck->execute();

            $paid = false;
            if ($Qcheck->numberOfRows() > 0) {
                while ($Qcheck->next()) {
                    if ($Qcheck->valueInt('orders_status_id') == $this->order_status) {
                        $paid = true;
                    }
                }
            }

            if ($paid === false) {
                if (osc_not_null(MODULE_PAYMENT_QUICKPAY_PROCESSING_ORDER_STATUS_ID)) {
                    osC_Order::process($_GET['invoice'], MODULE_PAYMENT_QUICKPAY_PROCESSING_ORDER_STATUS_ID, 'QuickPay Processing Transaction');
                }
            }
        }
        unset($_SESSION['prepOrderID']);
    }

    /**
     * Callback handler for quickpay callbacks
     */
    function callback() {
        global $osC_Database, $osC_Currencies;

        $requestBody = file_get_contents("php://input");
        $key = MODULE_PAYMENT_QUICKPAY_PRIVATE_KEY;
        $checksum = hash_hmac("sha256", $requestBody, $key);

        if ($checksum == $_SERVER["HTTP_QUICKPAY_CHECKSUM_SHA256"]) {
            $request = json_decode($requestBody);
            $operation = end($request->operations);
            $order_id = $request->order_id;

            $prefix = MODULE_PAYMENT_QUICKPAY_ORDER_PREFIX;
            if (isset($prefix)) {
                $order_id = substr($order_id, strlen($prefix));
            }

            $amount = ($operation->amount / 100.0);

            // Get the original amount so we can calculate the fee
            global $osC_Database;

            $class_total = "total";
            $Qorder = $osC_Database->query('select value from :table_orders_total where orders_id = :orders_id AND class=:class');
            $Qorder->bindTable(':table_orders_total', TABLE_ORDERS_TOTAL);
            $Qorder->bindInt(':orders_id', $order_id);
            $Qorder->bindValue(':class', $class_total);
            $Qorder->execute();

            $amount_orig = $Qorder->valueInt('value');

            $fee = $amount - $amount_orig;

            if ($request->accepted && $operation->type == 'authorize' && $operation->qp_status_code == "20000") {
                if ($fee > 0) {
                    $title = 'Payment Fee:';
                    $text = $osC_Currencies->format($fee);
                    $value = $fee;
                    $class = "payment_fee";
                    $sort_order = 70;

                    $Qordertotal = $osC_Database->query('insert into :table_orders_total (orders_id, title, text, value, class,sort_order) values (:orders_id, :title, :text, :value, :class, :sort_order)');
                    $Qordertotal->bindTable(':table_orders_total', TABLE_ORDERS_TOTAL);
                    $Qordertotal->bindInt(':orders_id', $order_id);
                    $Qordertotal->bindValue(':title', $title);
                    $Qordertotal->bindValue(':text', $text);
                    $Qordertotal->bindValue(':value', $value);
                    $Qordertotal->bindValue(':class', $class);
                    $Qordertotal->bindInt(':sort_order', $sort_order);
                    $Qordertotal->execute();

                    $class_total = "total";
                    $text = "<b>" . $osC_Currencies->format($amount) . "</b>";

                    $Qupdate = $osC_Database->query('update :table_orders_total set value = :value,text=:text where orders_id = :orders_id AND class=:class');
                    $Qupdate->bindTable(':table_orders_total', TABLE_ORDERS_TOTAL);
                    $Qupdate->bindValue(':value', $amount);
                    $Qupdate->bindValue(':text', $text);
                    $Qupdate->bindInt(':orders_id', $order_id);
                    $Qupdate->bindValue(':class', $class_total);
                    $Qupdate->execute();
                }

                $order_status = MODULE_PAYMENT_QUICKPAY_ORDER_STATUS_ID;
                $transaction = $request->id;

                $comments .= '\n QuickPay Transaction ID [' . $transaction . ']';
                osC_Order::process((int) $order_id, $order_status, $comments);
            }
        }
    }

    function sign($params, $api_key) {
        ksort($params);
        $base = implode(" ", $params);

        return hash_hmac("sha256", $base, $api_key);
    }

}

?>
